@@ -19,12 +19,13 @@
 #endif
 
 #include "btm.h"
+#include "iol.h"
+#include "ml.h"
+#include "il.h"
+#include <GL/glu.h>
 #include <stdlib.h>
 #include <string.h>
 #include <libgen.h>
-#include <IL/ilut.h>
-#include "btmio.h"
-#include "ml.h"
 
 static
 void
@@ -44,103 +45,102 @@ btmDeleteDisplayLists(BTMmodel *model);
 
 static
 void
-btmLoadString(char **name, BTMFILE *stream)
+btmLoadString(char **name, IOLFILE *stream)
 {
 	char *line = NULL;
-	size_t n;
+	unsigned int n;
 	char *sname;
 
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sname = strchr(line, ' ');
 	sname++;
-	*name = malloc((strlen(sname) + 1) * sizeof(char));
-	strcpy(*name, sname);
+	*name = strdup(sname);
 	free(line);
 }
 
 static
 void
-btmLoadFace(BTMface *face, BTMFILE *stream)
+btmLoadFace(BTMface *face, IOLFILE *stream)
 {
 	char *line = NULL;
-	size_t n;
+	unsigned int n;
 	unsigned int i;
 
 	// Skip {
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse mat
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tmat %i", &face->mat);
 	free(line);
 
 	// Parse verts
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tverts %i %i %i", &face->verts[0], &face->verts[1], &face->verts[2]);
 	free(line);
 
 	// Skip {
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse uv
 	for(i = 0; i < 3; i++)
 	{
-		btmgetline(&line, &n, stream);
+		iolgetline(&line, &n, stream);
 		sscanf(line, "\t\t\t\t\tuv %f %f %f", &face->uv[i][0], &face->uv[i][1], &face->uv[i][2]);
 		free(line);
 	}
 
 	// Skip }
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 	
 	// Parse no
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tno %f %f %f", &face->no[0], &face->no[1], &face->no[2]);
 	free(line);
 
 	// Skip }
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 }
 
 static
 void
-btmLoadVertex(BTMvertex *vertex, BTMFILE *stream)
+btmLoadVertex(BTMvertex *vertex, IOLFILE *stream)
 {
 	char *line = NULL;
-	size_t n;
+	unsigned int n;
 
 	// Skip {
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse co
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tco %f %f %f", &vertex->co[0], &vertex->co[1], &vertex->co[2]);
 	free(line);
 
 	// Parse no
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tno %f %f %f", &vertex->no[0], &vertex->no[1], &vertex->no[2]);
 	free(line);
 
 	// Skip }
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 }
 
 static
 void
-btmLoadMaterial(BTMmaterial *material, BTMFILE *stream)
+btmLoadMaterial(BTMmaterial *material, IOLFILE *stream)
 {
 	char *line = NULL;
-	size_t n;
+	unsigned int n;
 
 	// Skip {
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse name
@@ -150,56 +150,56 @@ btmLoadMaterial(BTMmaterial *material, BTMFILE *stream)
 	btmLoadString(&material->texture, stream);
 
 	// Parse alpha
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\talpha %f", &material->alpha);
 	free(line);
 
 	// Parse emit
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\temit %f", &material->emit);
 	free(line);
 
 	// Parse ambient
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tambient %f %f %f", &material->ambient[0], &material->ambient[1], &material->ambient[2]);
 	free(line);
 
 	// Parse diffuse
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tdiffuse %f %f %f", &material->diffuse[0], &material->diffuse[1], &material->diffuse[2]);
 	free(line);
 
 	// Parse specular
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\t\t\tspecular %f %f %f", &material->specular[0], &material->specular[1], &material->specular[2]);
 	free(line);
 
 	// Skip }
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 }
 
 static
 void
-btmLoadMesh(BTMmesh *mesh, BTMFILE *stream)
+btmLoadMesh(BTMmesh *mesh, IOLFILE *stream)
 {
 	char *line = NULL;
-	size_t n;
+	unsigned int n;
 	unsigned int i;
 
 	// Skip begin mesh
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 	
 	// Parse name
 	btmLoadString(&mesh->name, stream);
 
 	// Skip begin materials
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse materials length
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\tlength %i", &mesh->nmaterials);
 	free(line);
 
@@ -209,16 +209,15 @@ btmLoadMesh(BTMmesh *mesh, BTMFILE *stream)
 		btmLoadMaterial(&mesh->materials[i], stream);
 
 	// Skip end
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
-
 	// Skip begin vertices
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse vertices length
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\tlength %i", &mesh->nvertices);
 	free(line);
 
@@ -228,15 +227,15 @@ btmLoadMesh(BTMmesh *mesh, BTMFILE *stream)
 		btmLoadVertex(&mesh->vertices[i], stream);
 
 	// Skip end
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Skip begin faces
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse faces length
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\t\t\tlength %i", &mesh->nfaces);
 	free(line);
 
@@ -246,24 +245,23 @@ btmLoadMesh(BTMmesh *mesh, BTMFILE *stream)
 		btmLoadFace(&mesh->faces[i], stream);
 
 	// Skip end
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
-
 	// Skip end
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 }
 
 static
 void
-btmLoadObject(BTMobject *object, BTMFILE *stream)
+btmLoadObject(BTMobject *object, IOLFILE *stream)
 {
 	char *line = NULL;
-	size_t n;
+	unsigned int n;
 
 	// Skip {
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 	
 	// Parse name
@@ -272,18 +270,18 @@ btmLoadObject(BTMobject *object, BTMFILE *stream)
 	btmLoadMesh(&object->mesh, stream);
 
 	// Skip }
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 }
 
 void
 btmLoad(BTMmodel *model, const char *filename)
 {
-	BTMFILE *stream;
-	stream = btmfopenread(filename);
+	IOLFILE *stream;
+	stream = iolfopenread(filename);
 	char *line;
 	char *tmp;
-	size_t n;
+	unsigned int n;
 	unsigned int i;
 
 	tmp = strdup(filename);
@@ -295,25 +293,24 @@ btmLoad(BTMmodel *model, const char *filename)
 	free(tmp);
 
 	// Skip begin objects
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
 	// Parse objects length
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	sscanf(line, "\tlength %i", &model->nobjects);
 	free(line);
 	model->objects = malloc(model->nobjects * sizeof(BTMobject));
-	model->textures = malloc(model->nobjects * sizeof(GLuint *));
 
 	// Load objects
 	for(i = 0; i < model->nobjects; i++)
 		btmLoadObject(&model->objects[i], stream);
 
 	// Skip end
-	btmgetline(&line, &n, stream);
+	iolgetline(&line, &n, stream);
 	free(line);
 
-	btmfclose(stream);
+	iolfclose(stream);
 
 	btmGenTextures(model);
 	btmGenDisplayLists(model);
@@ -371,15 +368,15 @@ btmSetMaterial(BTMmaterial *material)
 	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material->ambient);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material->diffuse);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material->specular);
-	//glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, material->ambient);
+//	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, material->ambient);
 }
 
 static
 void
 btmGenDisplayLists(BTMmodel *model)
 {
-	size_t i, j, k, vertindex;
-	model->displaylists = malloc(model->nobjects * sizeof(GLuint *));
+	unsigned int i, j, k, vertindex;
+	model->displaylists = malloc(model->nobjects * sizeof(GLuint));
 	for(k = 0; k < model->nobjects; k++)
 	{
 		model->displaylists[k] = glGenLists(1);
@@ -407,10 +404,9 @@ static
 void
 btmDeleteDisplayLists(BTMmodel *model)
 {
-	size_t i;
-	for(i = 0; i < model->nobjects; i++)
-		glDeleteLists(model->displaylists[i], 1);
-
+	unsigned int k;
+	for(k = 0; k < model->nobjects; k++)
+		glDeleteLists(model->displaylists[k], 1);
 	free(model->displaylists);
 }
 
@@ -418,24 +414,27 @@ static
 void
 btmGenTextures(BTMmodel *model)
 {
-	unsigned int i, j;
-	char *data;
-	ILuint image;
+	unsigned int i, j, width, height;
+	char *image;
+	GLubyte *pixels;
+	GLenum format;
 
 	model->textures = malloc(model->nobjects * sizeof(GLuint *));
 	for(j = 0; j < model->nobjects; j++)
 	{
 		model->textures[j] = malloc(model->objects[j].mesh.nmaterials * sizeof(GLuint));
+		glGenTextures(model->objects[j].mesh.nmaterials, model->textures[j]);
 		for(i = 0; i < model->objects[j].mesh.nmaterials; i++)
 		{
-			ilGenImages(1, &image);
-			ilBindImage(image);
-			asprintf(&data, "%s/%s", model->path, model->objects[j].mesh.materials[i].texture);
-			ilLoadImage(data);
-			free(data);
-			model->textures[j][i] = ilutGLBindTexImage();
-			ilutGLBuildMipmaps();
-			ilDeleteImages(1, &image);
+			asprintf(&image, "%s/%s", model->path, model->objects[j].mesh.materials[i].texture);
+			glBindTexture(GL_TEXTURE_2D, model->textures[j][i]);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+			ilTexLoadMIME(image, &pixels, &width, &height, &format);
+			gluBuild2DMipmaps(GL_TEXTURE_2D, format, width, height, format, GL_UNSIGNED_BYTE, pixels);
+			free(pixels);
 		}
 	}
 }
@@ -456,13 +455,11 @@ btmDeleteTextures(BTMmodel *model)
 void
 btmRender(const BTMmodel *model)
 {
-	unsigned int i;
-	for(i = 0; i < model->nobjects; i++)
-		glCallList(model->displaylists[i]);
+	glCallLists(model->nobjects, GL_UNSIGNED_INT, model->displaylists);
 }
 
 void
-btmRenderWithCamera(const RILcamera *camera, const BTMmodel *model)
+btmRenderWithCamera(const FM_Camera *camera, const BTMmodel *model)
 {
 	unsigned int vertindex, i, j, k;
 	float triangle[3][3];
@@ -476,7 +473,7 @@ btmRenderWithCamera(const RILcamera *camera, const BTMmodel *model)
 				vertindex = model->objects[k].mesh.faces[j].verts[i];
 				mlPointCopy(model->objects[k].mesh.vertices[vertindex].co, triangle[i]);
 			}
-			if(rilCameraFrustumTriangle(camera, &triangle[0][0]))
+			if(FM_CameraFrustumTriangle(camera, &triangle[0][0]))
 			{
 				btmSetMaterial(&model->objects[k].mesh.materials[model->objects[k].mesh.faces[j].mat]);
 				glBindTexture(GL_TEXTURE_2D, model->textures[k][model->objects[k].mesh.faces[j].mat]);
