@@ -15,9 +15,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "machine.h"
-#include <math.h>
-#include "rilmedia.h"
 #include "ml.h"
+#include "sl.h"
+#include <math.h>
+#include <GL/gl.h>
 
 static
 void
@@ -80,29 +81,44 @@ void
 FM_ODE_FreeMachine(FM_Machine *machine)
 {
 	unsigned int i;
+	for(i = 0; i < 4; i++)
+		dJointDestroy(machine->joint[i]);
 	for(i = 0; i < 5; i++)
 		dGeomDestroy(machine->geom[i]);
+	for(i = 0; i < 5; i++)
+		dBodyDestroy(machine->body[i]);
 }
 
 static
 void
-FM_AL_LoadMachine(FM_Machine *machine, const char *engine, const float *position)
+FM_AL_LoadMachine(FM_Machine *machine, const char *enginefile, const float *position)
 {
-	machine->buffer = rilMediaALCreateBufferFromOgg(engine);
-	alGenSources(1, &machine->source);
-	alSourcei(machine->source, AL_BUFFER, machine->buffer);
-	alSourcei(machine->source, AL_LOOPING, AL_TRUE);
-	alSourcefv(machine->source, AL_POSITION, position);
-	alSourcef(machine->source, AL_GAIN, 10);
-	alSourcePlay(machine->source);
+	ALuint engine;
+	ALvoid *data;
+	ALsizei size, freq;
+	ALenum format;
+
+	alGenBuffers(1, &engine);
+	slBufferLoadMIME(enginefile, &size, &freq, &data, &format);
+	alBufferData(engine, format, data, size, freq);
+	free(data);
+	alGenSources(1, &machine->engine);
+	alSourcei(machine->engine, AL_BUFFER, engine);
+	alSourcei(machine->engine, AL_LOOPING, AL_TRUE);
+	alSourcefv(machine->engine, AL_POSITION, position);
+	alSourcef(machine->engine, AL_GAIN, 10);
+	alSourcePlay(machine->engine);
 }
 
 static
 void
 FM_AL_FreeMachine(FM_Machine *machine)
 {
-	alDeleteSources(1, &machine->source);
-	alDeleteBuffers(1, &machine->buffer);
+	ALuint engine;
+
+	alGetSourcei(machine->engine, AL_BUFFER, (int*)&engine);
+	alDeleteSources(1, &machine->engine);
+	alDeleteBuffers(1, &engine);
 }
 
 FM_Machine *
@@ -134,7 +150,7 @@ FM_LoadMachine(dWorldID world, dSpaceID space, float cfgspeed, float cfgsteer, c
 void
 FM_FreeMachine(FM_Machine *machine)
 {
-	alSourceStop(machine->source);
+	alSourceStop(machine->engine);
 
 	FM_AL_FreeMachine(machine);
 
@@ -186,7 +202,7 @@ FM_ControlMachine(FM_Machine *machine, int up, int down, int left, int right)
 }
 
 bool
-FM_UpdateMachine(FM_Machine *machine, const RILcamera *camera)
+FM_UpdateMachine(FM_Machine *machine, const FM_Camera *camera)
 {
 	const dReal *p;
 	float v;
@@ -196,18 +212,23 @@ FM_UpdateMachine(FM_Machine *machine, const RILcamera *camera)
 	v = 0;
 	for(i = 0; i < 4; i++)
 		v += mlVectorNormFunc(dBodyGetAngularVel(machine->body[i + 1]));
-	alSourcef(machine->source, AL_PITCH, (mlClampf(v / 80.0, 0, 1.5) + 0.5));
+	alSourcef(machine->engine, AL_PITCH, (mlClamp(v / 80.0, 0, 1.5) + 0.5));
 
 	p = dBodyGetPosition(machine->body[0]);
-	alSourcefv(machine->source, AL_POSITION, p);
-	frustum = rilCameraFrustumPoint(camera, p);
+	alSourcefv(machine->engine, AL_POSITION, p);
+	frustum = FM_CameraFrustumPoint(camera, p);
+//	p = dBodyGetLinearVel(machine->body[0]);
+//	alSourcefv(machine->engine, AL_VELOCITY, p);
+//	p = dBodyGetRotation(machine->body[0]);
+//	alSource3f(machine->engine, AL_DIRECTION, -p[2], -p[6], -p[10]);
+
 	return frustum;
 }
 
 void
 FM_RenderMachine(FM_Machine *machine)
 {
-	size_t i;
+	unsigned int i;
 	BTMmodel *model;
 	const dReal *p;
 	const dReal *r;
