@@ -17,16 +17,23 @@
 #ifndef MAIN_H
 #define MAIN_H
 
-#include <SDL/SDL.h>
+#include "freemachinestypes.h"
+#include <stdbool.h>
+#include "btm.h"
+#include <X11/Xlib.h>
 #include <GL/gl.h>
 #include <AL/al.h>
-#include <stdbool.h>
-#include "rilfont.h"
-#include "btm.h"
+#include <ode/ode.h>
+#undef PACKAGE
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+#undef VERSION
+#include <ft2build.h>
+#include FT_FREETYPE_H
 
-#define FM_SCREEN_WIDTH 640
-#define FM_SCREEN_HEIGHT 480
-#define FM_SCREEN_BPP 32
 #define FM_FONT PACKAGE_DATADIR "/VeraBd.ttf"
 #define FM_MENU_BACKGROUND PACKAGE_DATADIR "/Analog-Wind-1.jpg"
 #define FM_MENU_SELECTOR PACKAGE_DATADIR "/selector.btm.bz2"
@@ -43,11 +50,13 @@ typedef enum
 	FM_PROJECTION_3D
 } FM_Projection;
 
-typedef struct
+struct FM_Game
 {
-	SDL_Surface *screen;
 	FM_Projection projection;
-	Uint8 *keys;
+	FT_Library freetype;
+	FT_Face face;
+
+	struct { unsigned int w, h; } screen;
 
 	unsigned int ntracknames;
 	const char **tracknames;
@@ -55,29 +64,41 @@ typedef struct
 	unsigned int nmachinenames;
 	const char **machinenames;
 
-	// Menu
-	GLuint menubackground;
-	RILfont *menufont;
-	BTMmodel menuselector;
-	ALuint menuaudiobuffers[1], menuaudiosources[1];
+	struct
+	{
+		GLuint background;
+		BTMmodel selector;
+		ALuint audiosources[1];
+	} menu;
 
-	// Options
-	unsigned int nmachines, nplayers;
-	unsigned int track;
-	unsigned int machine;
-} FM_Game;
+	struct
+	{
+		unsigned int nmachines, nplayers;
+		unsigned int track, machine;
+	} options;
 
-void
-FM_EnterMain();
+	struct
+	{
+		dWorldID world;
+		dSpaceID space;
+		dJointGroupID contactgroup;
+		FM_Track *track;
+		struct { unsigned int track; } cache;
+	} play;
 
-void
-FM_ExitMain();
+};
 
 void
 FM_SetUpdateHandler(bool (*handler)(FM_Game *));
 
 void
-FM_SetEventHandler(bool (*handler)(FM_Game *, SDL_Event *));
+FM_SetKeyPressEventHandler(bool (*handler)(FM_Game *, XEvent *));
+
+void
+FM_SetKeyReleaseEventHandler(bool (*handler)(FM_Game *, XEvent *));
+
+void
+FM_SetExitHandler(void (*handler)(FM_Game *));
 
 void
 FM_GL_2D(FM_Game *game);
