@@ -15,44 +15,38 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "menuedit.h"
-#include <ode/ode.h>
-#include <GL/glu.h>
-#include "main.h"
 #include "menumain.h"
+#include "main.h"
+#include "camera.h"
 #include "machine.h"
 #include "track.h"
-#include "rilcamera.h"
-#include "rilfont.h"
-#include "list.h"
-#include "set.h"
+#include "chain.h"
 #include "ml.h"
+#include "cl.h"
+#include <X11/keysym.h>
+#include <GL/glu.h>
 
-dWorldID world;
-dSpaceID space;
-dJointGroupID contactgroup;
+static FM_Camera camera;
+static bool keys[4];
 
-static RILfont *font;
-static RILcamera camera;
-static FM_Track *track;
+static chain *machines;
+static chain *activemachines;
+static chain *checkpoints;
 
-static list *machines;
-static set *activemachines;
-static list *checkpoints; // Used for generator.
-
-static const char *fontfilename = PACKAGE_DATADIR "/VeraBd.ttf";
-
-#define MAX_CONTACTS 1000
+#define CONTACTS 1024
 
 static
 void
 FM_NearCallback(void *data, dGeomID o1, dGeomID o2)
 {
 	dBodyID b1, b2;
-	dContact contact[MAX_CONTACTS];
+	dContact contact[CONTACTS];
 	dJointID c;
-	unsigned int i;
-	int n;
+	int i, n;
 	void *d1, *d2;
+	FM_Game *game;
+
+	game = data;
 
 	b1 = dGeomGetBody(o1);
 	b2 = dGeomGetBody(o2);
@@ -62,10 +56,10 @@ FM_NearCallback(void *data, dGeomID o1, dGeomID o2)
 	d1 = dGeomGetData(o1);
 	d2 = dGeomGetData(o2);
 
-	if(set_contains(machines, d1) && set_contains(machines, d2) && (!set_contains(activemachines, d1) || !set_contains(activemachines, d2)))
+	if(chain_contains(machines, d1) && chain_contains(machines, d2) && (!chain_contains(activemachines, d1) || !chain_contains(activemachines, d2)))
 		return;
 
-	n = dCollide(o1, o2, MAX_CONTACTS, &contact[0].geom, sizeof(dContact));
+	n = dCollide(o1, o2, CONTACTS, &contact[0].geom, sizeof(dContact));
 	if(n > 0)
 	{
 		for(i = 0; i < n; i++)
@@ -77,7 +71,7 @@ FM_NearCallback(void *data, dGeomID o1, dGeomID o2)
 			contact[i].surface.slip2 = 0.01;
 			contact[i].surface.soft_erp = 0.5;
 			contact[i].surface.soft_cfm = 0.3;
-			c = dJointCreateContact(world,contactgroup,&contact[i]);
+			c = dJointCreateContact(game->play.world, game->play.contactgroup, &contact[i]);
 			dJointAttach(c, dGeomGetBody(contact[i].geom.g1), dGeomGetBody(contact[i].geom.g2));
 		}
 	}
@@ -87,14 +81,14 @@ static
 void
 FM_ActivateMachinesScanner(FM_Machine *machine)
 {
-	activemachines = set_add(activemachines, machine);
+	activemachines = chain_append(activemachines, machine);
 }
 
 static
 void
 FM_ActivateMachines()
 {
-	list_scan(machines, (list_scanner)FM_ActivateMachinesScanner);
+	chain_scan(machines, (scanner)FM_ActivateMachinesScanner);
 }
 
 static
@@ -113,59 +107,74 @@ FM_EnterMenuEdit(FM_Game *game)
 	float pos[3];
 	unsigned int i;
 
-	alSourceStop(game->menuaudiosources[FM_GAME_MENU_MUSIC]);
+	alSourceStop(game->menu.audiosources[FM_GAME_MENU_MUSIC]);
 
-	font = rilFontLoad(fontfilename, RIL_FONT_BITMAP);
-
-	world = dWorldCreate();
-	space = dHashSpaceCreate(0);
-	contactgroup = dJointGroupCreate(0);
-	dWorldSetGravity(world, 0, -9.8, 0);
+	memset(keys, 0, sizeof(bool) * 16);
 
 	mlPointLoad(camera.object, 0, 0, 0);
 
-	track = FM_LoadTrack(space, game->tracknames[game->track]);
+	if(!game->play.track)
+	{
+		game->play.track = FM_LoadTrack(game->play.space, game->tracknames[game->options.track]);
+		game->play.cache.track = game->options.track;
+	}
+	else
+	{
+		if(game->options.track != game->play.cache.track)
+		{
+			FM_FreeTrack(game->play.track);
+			game->play.track = FM_LoadTrack(game->play.space, game->tracknames[game->options.track]);
+			game->play.cache.track = game->options.track;
+		}
+	}
 
 	machines = activemachines = checkpoints = NULL;
 
-	for(i = 0; i < track->ncheckpoints; i++)
-		checkpoints = list_append(checkpoints, FM_GenCheckPoint(&track->checkpoints[i * 3]));
+	for(i = 0; i < game->play.track->ncheckpoints; i++)
+		checkpoints = chain_append(checkpoints, FM_GenCheckPoint(&game->play.track->checkpoints[i * 3]));
 
 	mlPointLoad(pos, 0, 5, 0);
-	machines = list_append(machines, FM_LoadMachine(world, space, 37.5, 20, pos, -90, game->machinenames[game->machine]));
+	machines = chain_append(machines, FM_LoadMachine(game->play.world, game->play.space, 37.5, 20, pos, -90, game->machinenames[game->options.machine]));
 
 	FM_ActivateMachines();
 
-	FM_SetEventHandler(FM_EventMenuEdit);
+	FM_SetKeyPressEventHandler(FM_KeyPressEventMenuEdit);
+	FM_SetKeyReleaseEventHandler(FM_KeyReleaseEventMenuEdit);
 	FM_SetUpdateHandler(FM_UpdateMenuEdit);
+	FM_SetExitHandler(FM_ExitMenuEdit);
 }
 
 void
 FM_ExitMenuEdit(FM_Game *game)
 {
-	rilFontUnload(font);
 
-	list_scan(machines, (list_scanner)FM_FreeMachine);
-	list_free(machines);
+	chain_scan(machines, (scanner)FM_FreeMachine);
+	chain_free(machines);
 
-	list_scan(checkpoints, (list_scanner)free);
-	list_free(checkpoints);
+	chain_free(activemachines);
 
-	FM_FreeTrack(track);
-
-	dJointGroupDestroy(contactgroup);
-	dSpaceDestroy(space);
-	dWorldDestroy(world);
+	chain_scan(checkpoints, (scanner)free);
+	chain_free(checkpoints);
 }
 
 static
 void
-FM_UpdateMachineScanner(FM_Machine *machine, const RILcamera *camera)
+FM_UpdateActiveMachines()
 {
-	if(!FM_UpdateMachine(machine, camera))
+	chain *c, *n;
+	FM_Machine *machine;
+
+	c = activemachines;
+	while(c)
 	{
-		activemachines = set_remove(activemachines, machine);
-		alSourceStop(machine->source);
+		machine = c->data;
+		n = c->next;
+		if(!FM_UpdateMachine(machine, &camera))
+		{
+			activemachines = chain_remove_chain(activemachines, c);
+			alSourceStop(machine->engine);
+		}
+		c = n;
 	}
 }
 
@@ -176,7 +185,6 @@ FM_RenderMenuEdit(FM_Game *game)
 	FM_Machine *machine;
 	float a[3], b[3];
 	const dReal *p, *v;
-	char text[2];
 
 	machine = activemachines->data;
 
@@ -195,29 +203,20 @@ FM_RenderMenuEdit(FM_Game *game)
 
 	glLoadIdentity();
 	gluLookAt(camera.object[0], 70, camera.object[2], camera.object[0], camera.object[1], camera.object[2], 0, 0, -1);
-	rilCameraGLFrustumLoad(&camera);
+	FM_CameraGLFrustumLoad(&camera);
 
-	list_user_scan(activemachines, (list_user_scanner)FM_UpdateMachineScanner, &camera);
-	FM_RenderTrack(track, &camera);
-	list_scan(activemachines, (list_scanner)FM_RenderMachine);
+	FM_UpdateActiveMachines();
+	FM_RenderTrack(game->play.track, &camera);
+	chain_scan(activemachines, (scanner)FM_RenderMachine);
 
-	if(list_length(checkpoints) >= 2)
+	if(chain_length(checkpoints) >= 2)
 	{
-		list *l;
+		chain *l;
 		glBegin(GL_LINE_STRIP);
 		for(l = checkpoints; l; l = l->next)
 			glVertex3fv(l->data);
 		glEnd();
 	}
-
-	FM_GL_2D(game);
-	rilFontFaceSize(font, 43);
-	sprintf(text, "%i", set_size(activemachines));
-	glRasterPos2i(0, 0);
-	rilFontRender(font, text);
-	FM_GL_3D(game);
-
-	SDL_GL_SwapBuffers();
 }
 
 static
@@ -227,7 +226,42 @@ FM_ControlActiveMachines(FM_Game *game)
 	FM_Machine *machine;
 
 	machine = activemachines->data;
-	FM_ControlMachine(machine, game->keys[SDLK_UP], game->keys[SDLK_DOWN], game->keys[SDLK_LEFT], game->keys[SDLK_RIGHT]);
+	FM_ControlMachine(machine, keys[0], keys[1], keys[2], keys[3]);
+}
+
+bool
+FM_KeyPressEventMenuEdit(FM_Game *game, XEvent *event)
+{
+	KeySym keysym;
+
+	keysym = XLookupKeysym(&event->xkey, 0);
+	switch(keysym)
+	{
+		case XK_Up: keys[0] = true; break;
+		case XK_Down: keys[1] = true; break;
+		case XK_Left: keys[2] = true; break;
+		case XK_Right: keys[3] = true; break;
+	}
+	return true;
+}
+
+bool
+FM_KeyReleaseEventMenuEdit(FM_Game *game, XEvent *event)
+{
+	KeySym keysym;
+
+	keysym = XLookupKeysym(&event->xkey, 0);
+	switch(keysym)
+	{
+		case XK_Up: keys[0] = false; break;
+		case XK_Down: keys[1] = false; break;
+		case XK_Left: keys[2] = false; break;
+		case XK_Right: keys[3] = false; break;
+		case XK_c: checkpoints = chain_append(checkpoints, FM_GenCheckPoint(dBodyGetPosition(((FM_Machine *)machines->data)->body[0]))); break; case XK_v: checkpoints = chain_remove(checkpoints, chain_length(checkpoints) - 1); break;
+		case XK_p: chain_scan(checkpoints, (scanner)mlPointPrint); break;
+		case XK_Escape: FM_ExitMenuEdit(game); FM_EnterMenuMain(game); break;
+	}
+	return true;
 }
 
 bool
@@ -241,46 +275,13 @@ FM_UpdateMenuEdit(FM_Game *game)
 
 	for(i = 0; i < 16; i++)
 	{
-		dSpaceCollide(space, NULL, &FM_NearCallback);
-		dWorldStep(world, 0.01);
-		dJointGroupEmpty(contactgroup);
+		dSpaceCollide(game->play.space, game, &FM_NearCallback);
+		dWorldStep(game->play.world, 0.01);
+		dJointGroupEmpty(game->play.contactgroup);
 	}
 
 	FM_RenderMenuEdit(game);
 
-	return false;
+	return true;
 }
 
-bool
-FM_EventMenuEdit(FM_Game *game, SDL_Event *event)
-{
-	bool exitrequest = false;
-	switch(event->type)
-	{
-		case SDL_KEYUP:
-			switch(event->key.keysym.sym)
-			{
-				case SDLK_c:
-					checkpoints = list_append(checkpoints, FM_GenCheckPoint(dBodyGetPosition(((FM_Machine *)machines->data)->body[0])));
-					break;
-				case SDLK_v:
-					checkpoints = list_remove(checkpoints, list_length(checkpoints) - 1);
-					break;
-				case SDLK_p:
-					list_scan(checkpoints, (list_scanner)mlPointPrint);
-					break;
-				case SDLK_ESCAPE:
-					FM_ExitMenuEdit(game);
-					FM_EnterMenuMain(game);
-					break;
-				default:
-					break;
-			}
-			break;
-		case SDL_QUIT:
-			FM_ExitMenuEdit(game);
-			exitrequest = true;
-			break;
-	}
-	return exitrequest;
-}
