@@ -17,8 +17,6 @@
 #ifndef MENUEDIT_H
 #define MENUEDIT_H
 
-#include <SDL/SDL.h>
-#include <stdbool.h>
 #include "main.h"
 
 void
@@ -31,6 +29,9 @@ bool
 FM_UpdateMenuEdit(FM_Game *game);
 
 bool
-FM_EventMenuEdit(FM_Game *game, SDL_Event *event);
+FM_KeyPressEventMenuEdit(FM_Game *game, XEvent *event);
+
+bool
+FM_KeyReleaseEventMenuEdit(FM_Game *game, XEvent *event);
 
 #endif
