@@ -15,44 +15,48 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "menuhelp.h"
-#include <GL/glu.h>
-#include "main.h"
 #include "menumain.h"
-#include "rilfont.h"
+#include "main.h"
+#include "cl.h"
+#include <stdlib.h>
+#include <X11/keysym.h>
+#include <GL/glu.h>
 #include "config.h"
 
 static const wchar_t *help[] = 
 {
-	L"A free machines game.",
-	L"Drive the machines in the various tracks.",
+	L"A free machines game",
+	L"Drive the machines in the various tracks",
 	L"Player 1 keys: up, down, left, right",
 	L"Player 2 keys: w, s, a, d",
 	L"Player 3 keys: i, k, j, l",
 	L"Player 4 keys: t, g, f, h",
-	L"Have fun.",
+	L"Have fun",
 };
 
 void
 FM_EnterMenuHelp(FM_Game *game)
 {
-	alSourcePlay(game->menuaudiosources[FM_GAME_MENU_MUSIC]);
+	alSourcePlay(game->menu.audiosources[FM_GAME_MENU_MUSIC]);
 
-	FM_SetEventHandler(FM_EventMenuHelp);
+	FM_SetKeyPressEventHandler(FM_KeyPressEventMenuHelp);
+	FM_SetKeyReleaseEventHandler(FM_KeyReleaseEventMenuHelp);
 	FM_SetUpdateHandler(FM_UpdateMenuHelp);
+	FM_SetExitHandler(FM_ExitMenuHelp);
 }
 
 void
 FM_ExitMenuHelp(FM_Game *game)
 {
-	alSourcePause(game->menuaudiosources[FM_GAME_MENU_MUSIC]);
+	alSourcePause(game->menu.audiosources[FM_GAME_MENU_MUSIC]);
 }
 
 static
 void
 FM_RenderMenuHelp(FM_Game *game)
 {
-	const float aspect = (float)game->screen->w / game->screen->h;
-	float bbox[6];
+	const float aspect = (float) game->screen.w / game->screen.h;
+	unsigned int metrics[2];
 	int i;
 
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -61,9 +65,9 @@ FM_RenderMenuHelp(FM_Game *game)
 	
 	gluLookAt(0, 0, 100, 0, 0, 0, 0, 1, 0);
 
-	glBindTexture(GL_TEXTURE_2D, game->menubackground);
+	glBindTexture(GL_TEXTURE_2D, game->menu.background);
 	glPushMatrix();
-		glTranslatef((float)rand() / RAND_MAX - 0.5, (float)rand() / RAND_MAX - 0.5, 0);
+		glTranslatef((float) rand() / RAND_MAX - 0.5, (float)rand() / RAND_MAX - 0.5, 0);
 		glBegin(GL_QUADS);
 			glTexCoord2i(0, 0); glVertex2f(-52 * aspect, -52);
 			glTexCoord2i(1, 0); glVertex2f(52 * aspect, -52);
@@ -72,52 +76,45 @@ FM_RenderMenuHelp(FM_Game *game)
 		glEnd();
 	glPopMatrix();
 
-	rilFontFaceSize(game->menufont, 3);
-	rilFontDepth(game->menufont, 1);
-	for(i = 0; i < sizeof(help) / sizeof(help[0]); i++)
+	glDisable(GL_TEXTURE_2D);
+	glPushMatrix();
+	glScalef(0.05, 0.05, 1);
+	for(i = 0; i < 7; i++)
 	{
-		rilFontBBoxWide(game->menufont, help[i], bbox);
-		glPushAttrib(GL_ALL_ATTRIB_BITS);
-		glDisable(GL_TEXTURE_2D);
+		clStringMetrics(game->face, help[i], &metrics[0], &metrics[1]);
 		glPushMatrix();
-		glTranslatef((float)rand() / RAND_MAX - 0.5, (float)rand() / RAND_MAX - 0.5, 0);
-		glTranslatef((bbox[0] - bbox[3]) / 2, (3 - i) * 10, 0);
-		rilFontRenderWide(game->menufont, help[i]);
+		glTranslatef((float) rand() / RAND_MAX * 20.0 - 10.0, (float) rand() / RAND_MAX * 20.0 - 10.0, 0);
+		glTranslatef(-(int) metrics[0] / 2, (3 - i) * 200, 1);
+		clRenderString(game->face, CL_FILL, help[i], 1);
 		glPopMatrix();
-		glPopAttrib();
-	}
+	}	
+	glPopMatrix();
+	glEnable(GL_TEXTURE_2D);
+}
 
-	SDL_GL_SwapBuffers();
+bool
+FM_KeyPressEventMenuHelp(FM_Game *game, XEvent *event)
+{
+	return true;
+}
+
+bool
+FM_KeyReleaseEventMenuHelp(FM_Game *game, XEvent *event)
+{
+	KeySym keysym;
+
+	keysym = XLookupKeysym(&event->xkey, 0);
+	switch(keysym)
+	{
+		case XK_Escape: FM_ExitMenuHelp(game); FM_EnterMenuMain(game); break;
+	}
+	return true;
 }
 
 bool
 FM_UpdateMenuHelp(FM_Game *game)
 {
 	FM_RenderMenuHelp(game);
-	return false;
+	return true;
 }
 
-bool
-FM_EventMenuHelp(FM_Game *game, SDL_Event *event)
-{
-	bool exitrequest = false;
-	switch(event->type)
-	{
-		case SDL_KEYUP:
-			switch(event->key.keysym.sym)
-			{
-				case SDLK_ESCAPE:
-					FM_ExitMenuHelp(game);
-					FM_EnterMenuMain(game);
-					break;
-				default:
-					break;
-			}
-			break;
-		case SDL_QUIT:
-			FM_ExitMenuHelp(game);
-			exitrequest = true;
-			break;
-	}
-	return exitrequest;
-}
