@@ -17,8 +17,6 @@
 #ifndef MENUHELP_H
 #define MENUHELP_H
 
-#include <SDL/SDL.h>
-#include <stdbool.h>
 #include "main.h"
 
 void
@@ -31,6 +29,9 @@ bool
 FM_UpdateMenuHelp(FM_Game *game);
 
 bool
-FM_EventMenuHelp(FM_Game *game, SDL_Event *event);
+FM_KeyPressEventMenuHelp(FM_Game *game, XEvent *event);
+
+bool
+FM_KeyReleaseEventMenuHelp(FM_Game *game, XEvent *event);
 
 #endif
