@@ -17,8 +17,6 @@
 #ifndef MENUMAIN_H
 #define MENUMAIN_H
 
-#include <SDL/SDL.h>
-#include <stdbool.h>
 #include "main.h"
 
 void
@@ -31,6 +29,9 @@ bool
 FM_UpdateMenuMain(FM_Game *game);
 
 bool
-FM_EventMenuMain(FM_Game *game, SDL_Event *event);
+FM_KeyPressEventMenuMain(FM_Game *game, XEvent *event);
+
+bool
+FM_KeyReleaseEventMenuMain(FM_Game *game, XEvent *event);
 
 #endif
