@@ -17,8 +17,6 @@
 #ifndef MENUOPTIONS_H
 #define MENUOPTIONS_H
 
-#include <SDL/SDL.h>
-#include <stdbool.h>
 #include "main.h"
 
 void
@@ -31,6 +29,9 @@ bool
 FM_UpdateMenuOptions(FM_Game *game);
 
 bool
-FM_EventMenuOptions(FM_Game *game, SDL_Event *event);
+FM_KeyPressEventMenuOptions(FM_Game *game, XEvent *event);
+
+bool
+FM_KeyReleaseEventMenuOptions(FM_Game *game, XEvent *event);
 
 #endif
