@@ -15,44 +15,38 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "menuplay.h"
-#include <ode/ode.h>
-#include <GL/glu.h>
-#include "main.h"
 #include "menumain.h"
+#include "main.h"
+#include "camera.h"
 #include "machine.h"
 #include "track.h"
-#include "rilcamera.h"
-#include "rilfont.h"
-#include "list.h"
-#include "set.h"
+#include "chain.h"
+#include "cl.h"
 #include "ml.h"
+#include <X11/keysym.h>
+#include <GL/glu.h>
 
-dWorldID world;
-dSpaceID space;
-dJointGroupID contactgroup;
+static FM_Camera camera;
 
-static RILfont *font;
-static RILcamera camera;
-static FM_Track *track;
+static chain *machines;
+static chain *activemachines;
+static bool keys[4][4];
 
-static list *machines;
-static set *activemachines;
-
-static const char *fontfilename = PACKAGE_DATADIR "/VeraBd.ttf";
-
-#define MAX_CONTACTS 1000
+#define CONTACTS 1024
 
 static
 void
 FM_NearCallback(void *data, dGeomID o1, dGeomID o2)
 {
+	FM_Game *game;
 	dBodyID b1, b2;
-	dContact contact[MAX_CONTACTS];
+	dContact contact[CONTACTS];
 	dJointID c;
-	unsigned int i;
-	int n;
+	int i, n;
 	void *d1, *d2;
 
+	game = data;
+
 	b1 = dGeomGetBody(o1);
 	b2 = dGeomGetBody(o2);
 	if(b1 && b2 && dAreConnectedExcluding(b1, b2, dJointTypeContact))
@@ -61,10 +55,10 @@ FM_NearCallback(void *data, dGeomID o1, dGeomID o2)
 	d1 = dGeomGetData(o1);
 	d2 = dGeomGetData(o2);
 
-	if(set_contains(machines, d1) && set_contains(machines, d2) && (!set_contains(activemachines, d1) || !set_contains(activemachines, d2)))
+	if(chain_contains(machines, d1) && chain_contains(machines, d2) && (!chain_contains(activemachines, d1) || !chain_contains(activemachines, d2)))
 		return;
 
-	n = dCollide(o1, o2, MAX_CONTACTS, &contact[0].geom, sizeof(dContact));
+	n = dCollide(o1, o2, CONTACTS, &contact[0].geom, sizeof(dContact));
 	if(n > 0)
 	{
 		for(i = 0; i < n; i++)
@@ -76,7 +70,7 @@ FM_NearCallback(void *data, dGeomID o1, dGeomID o2)
 			contact[i].surface.slip2 = 0.01;
 			contact[i].surface.soft_erp = 0.5;
 			contact[i].surface.soft_cfm = 0.3;
-			c = dJointCreateContact(world,contactgroup,&contact[i]);
+			c = dJointCreateContact(game->play.world, game->play.contactgroup, &contact[i]);
 			dJointAttach(c, dGeomGetBody(contact[i].geom.g1), dGeomGetBody(contact[i].geom.g2));
 		}
 	}
@@ -86,14 +80,14 @@ static
 void
 FM_ActivateMachinesScanner(FM_Machine *machine)
 {
-	activemachines = set_add(activemachines, machine);
+	activemachines = chain_append(activemachines, machine);
 }
 
 static
 void
 FM_ActivateMachines()
 {
-	list_scan(machines, (list_scanner)FM_ActivateMachinesScanner);
+	chain_scan(machines, (scanner)FM_ActivateMachinesScanner);
 }
 
 void
@@ -102,57 +96,69 @@ FM_EnterMenuPlay(FM_Game *game)
 	float pos[3];
 	unsigned int i;
 
-	alSourceStop(game->menuaudiosources[FM_GAME_MENU_MUSIC]);
+	alSourceStop(game->menu.audiosources[FM_GAME_MENU_MUSIC]);
 
-	font = rilFontLoad(fontfilename, RIL_FONT_BITMAP);
-
-	world = dWorldCreate();
-	space = dHashSpaceCreate(0);
-	contactgroup = dJointGroupCreate(0);
-	dWorldSetGravity(world, 0, -9.8, 0);
+	memset(keys, 0, sizeof(bool) * 16);
+	machines = activemachines = NULL;
 
 	mlPointLoad(camera.object, 0, 0, 0);
 	alListener3f(AL_POSITION, camera.object[0], 75, camera.object[2]);
 
-	track = FM_LoadTrack(space, game->tracknames[game->track]);
-
-	machines = activemachines = NULL;
-
-	for(i = 0; i < game->nmachines; i++)
+	if(!game->play.track)
 	{
-		mlPointLoad(pos, (i - game->nmachines / 2.0) * 5.0, 5, 0);
-		machines = list_append(machines, FM_LoadMachine(world, space, 75, 20, pos, -90, game->machinenames[game->machine]));
+		game->play.track = FM_LoadTrack(game->play.space, game->tracknames[game->options.track]);
+		game->play.cache.track = game->options.track;
+	}
+	else
+	{
+		if(game->options.track != game->play.cache.track)
+		{
+			FM_FreeTrack(game->play.track);
+			game->play.track = FM_LoadTrack(game->play.space, game->tracknames[game->options.track]);
+			game->play.cache.track = game->options.track;
+		}
+	}
+
+	for(i = 0; i < game->options.nmachines; i++)
+	{
+		mlPointLoad(pos, (i - game->options.nmachines / 2.0) * 5.0, 5, 0);
+		machines = chain_append(machines, FM_LoadMachine(game->play.world, game->play.space, 75, 20, pos, -90, game->machinenames[game->options.machine]));
 	}
 
 	FM_ActivateMachines();
 
-	FM_SetEventHandler(FM_EventMenuPlay);
+	FM_SetKeyPressEventHandler(FM_KeyPressEventMenuPlay);
+	FM_SetKeyReleaseEventHandler(FM_KeyReleaseEventMenuPlay);
 	FM_SetUpdateHandler(FM_UpdateMenuPlay);
+	FM_SetExitHandler(FM_ExitMenuPlay);
 }
 
 void
 FM_ExitMenuPlay(FM_Game *game)
 {
-	rilFontUnload(font);
-
-	list_scan(machines, (list_scanner)FM_FreeMachine);
-	list_free(machines);
-
-	FM_FreeTrack(track);
-
-	dJointGroupDestroy(contactgroup);
-	dSpaceDestroy(space);
-	dWorldDestroy(world);
+	chain_scan(machines, (scanner)FM_FreeMachine);
+	chain_free(machines);
+	chain_free(activemachines);
 }
 
 static
 void
-FM_UpdateMachineScanner(FM_Machine *machine, const RILcamera *camera)
+FM_UpdateActiveMachines()
 {
-	if(!FM_UpdateMachine(machine, camera))
+	chain *c, *n;
+	FM_Machine *machine;
+
+	c = activemachines;
+	while(c)
 	{
-		activemachines = set_remove(activemachines, machine);
-		alSourceStop(machine->source);
+		machine = c->data;
+		n = c->next;
+		if(!FM_UpdateMachine(machine, &camera))
+		{
+			activemachines = chain_remove_chain(activemachines, c);
+			alSourceStop(machine->engine);
+		}
+		c = n;
 	}
 }
 
@@ -161,7 +167,7 @@ FM_Machine *
 FM_FirstMachine()
 {
 	FM_Machine *machine, *firstmachine;
-	list *l;
+	chain *l;
 
 	firstmachine = NULL;
 	if(activemachines)
@@ -184,7 +190,7 @@ FM_RenderMenuPlay(FM_Game *game)
 	FM_Machine *firstmachine;
 	float a[3], b[3];
 	const dReal *p, *v;
-	char text[2];
+	wchar_t text[2];
 
 	firstmachine = FM_FirstMachine();
 
@@ -207,25 +213,26 @@ FM_RenderMenuPlay(FM_Game *game)
 	glLoadIdentity();
 
 	gluLookAt(camera.object[0], 70, camera.object[2], camera.object[0], camera.object[1], camera.object[2], 0, 0, -1);
-	rilCameraGLFrustumLoad(&camera);
+	FM_CameraGLFrustumLoad(&camera);
 
-	list_user_scan(activemachines, (list_user_scanner)FM_UpdateMachineScanner, &camera);
-	FM_RenderTrack(track, &camera);
-	list_scan(activemachines, (list_scanner)FM_RenderMachine);
+	FM_UpdateActiveMachines();
+	FM_RenderTrack(game->play.track, &camera);
+	chain_scan(activemachines, (scanner)FM_RenderMachine);
 
-	FM_GL_2D(game);
-	rilFontFaceSize(font, 43);
-	sprintf(text, "%i", set_size(activemachines));
-	glRasterPos2i(0, 0);
-	rilFontRender(font, text);
-	FM_GL_3D(game);
-
-	SDL_GL_SwapBuffers();
+	swprintf(text, 2, L"%i", chain_length(activemachines));
+	glDisable(GL_TEXTURE_2D);
+	glPushMatrix();
+	glTranslatef(camera.object[0] - 4.0 * (float) game->screen.w / game->screen.h, 60, camera.object[2] + 4.0);
+	glRotatef(-90, 1, 0, 0);
+	glScalef(0.02, 0.02, 1);
+	clRenderString(game->face, CL_OUTLINE, text, 1);
+	glPopMatrix();
+	glEnable(GL_TEXTURE_2D);
 }
 
 static
 void
-FM_ComputerControlMachine(FM_Machine *machine)
+FM_ComputerControlMachine(FM_Game *game, FM_Machine *machine)
 {
 	float position[3], target[3], direction[3], orientation[3], plane[4], angle, variation, distance;
 	const dReal *r;
@@ -234,7 +241,7 @@ FM_ComputerControlMachine(FM_Machine *machine)
 	variation = (float)rand() / RAND_MAX;
 
 	mlPointCopy(dBodyGetPosition(machine->body[0]), position);
-	mlPointCopy(&track->checkpoints[(machine->checkpoint + 1) % (track->ncheckpoints) * 3], target);
+	mlPointCopy(&game->play.track->checkpoints[(machine->checkpoint + 1) % (game->play.track->ncheckpoints) * 3], target);
 
 	mlVectorSub(target, position, direction);
 
@@ -244,7 +251,7 @@ FM_ComputerControlMachine(FM_Machine *machine)
 	{
 		machine->checkpoint++;
 
-		mlPointCopy(&track->checkpoints[(machine->checkpoint + 1) % (track->ncheckpoints) * 3], target);
+		mlPointCopy(&game->play.track->checkpoints[(machine->checkpoint + 1) % (game->play.track->ncheckpoints) * 3], target);
 
 		mlVectorSub(target, position, direction);
 
@@ -289,12 +296,12 @@ FM_ComputerControlMachine(FM_Machine *machine)
 
 static
 void
-FM_HumanControlMachine(FM_Machine *machine, bool up, bool down, bool left, bool right)
+FM_HumanControlMachine(FM_Game *game, FM_Machine *machine, const bool *keys)
 {
 	float position[3], target[3], direction[3];
 
 	mlPointCopy(dBodyGetPosition(machine->body[0]), position);
-	mlPointCopy(&track->checkpoints[(machine->checkpoint + 1) % (track->ncheckpoints) * 3], target);
+	mlPointCopy(&game->play.track->checkpoints[(machine->checkpoint + 1) % (game->play.track->ncheckpoints) * 3], target);
 
 	mlVectorSub(target, position, direction);
 
@@ -304,51 +311,91 @@ FM_HumanControlMachine(FM_Machine *machine, bool up, bool down, bool left, bool
 	{
 		machine->checkpoint++;
 
-		mlPointCopy(&track->checkpoints[(machine->checkpoint + 1) % (track->ncheckpoints) * 3], target);
+		mlPointCopy(&game->play.track->checkpoints[(machine->checkpoint + 1) % (game->play.track->ncheckpoints) * 3], target);
 
 		mlVectorSub(target, position, direction);
 
 		mlVectorNorm(direction, &machine->checkpointdistance);
 	}
 
-	FM_ControlMachine(machine, up, down, left, right);
+	FM_ControlMachine(machine, keys[0], keys[1], keys[2], keys[3]);
 }
 
 static
 void
 FM_ControlActiveMachines(FM_Game *game)
 {
-	list *l;
+	chain *l;
 	unsigned int i;
 	FM_Machine *machine;
-
 	for(i = 0, l = machines; l; l = l->next, i++)
 	{
 		machine = l->data;
-		if(set_contains(activemachines, machine))
+		if(chain_contains(activemachines, machine))
 		{
-			if(i < game->nplayers)
-				switch(i)
-				{
-					case 0:
-						FM_HumanControlMachine(machine, game->keys[SDLK_UP], game->keys[SDLK_DOWN], game->keys[SDLK_LEFT], game->keys[SDLK_RIGHT]);
-						break;
-					case 1:
-						FM_HumanControlMachine(machine, game->keys[SDLK_w], game->keys[SDLK_s], game->keys[SDLK_a], game->keys[SDLK_d]);
-						break;
-					case 2:
-						FM_HumanControlMachine(machine, game->keys[SDLK_i], game->keys[SDLK_k], game->keys[SDLK_j], game->keys[SDLK_l]);
-						break;
-					case 3:
-						FM_HumanControlMachine(machine, game->keys[SDLK_t], game->keys[SDLK_g], game->keys[SDLK_f], game->keys[SDLK_h]);
-						break;
-				}
-			else
-				FM_ComputerControlMachine(machine);
+			i < game->options.nplayers ? FM_HumanControlMachine(game, machine, keys[i]) : FM_ComputerControlMachine(game, machine);
 		}
 	}
 }
 
+bool
+FM_KeyPressEventMenuPlay(FM_Game *game, XEvent *event)
+{
+	KeySym keysym;
+
+	keysym = XLookupKeysym(&event->xkey, 0);
+	switch(keysym)
+	{
+		case XK_Up: keys[0][0] = true; break;
+		case XK_Down: keys[0][1] = true; break;
+		case XK_Left: keys[0][2] = true; break;
+		case XK_Right: keys[0][3] = true; break;
+		case XK_w: keys[1][0] = true; break;
+		case XK_s: keys[1][1] = true; break;
+		case XK_a: keys[1][2] = true; break;
+		case XK_d: keys[1][3] = true; break;
+		case XK_i: keys[2][0] = true; break;
+		case XK_k: keys[2][1] = true; break;
+		case XK_j: keys[2][2] = true; break;
+		case XK_l: keys[2][3] = true; break;
+		case XK_t: keys[3][0] = true; break;
+		case XK_g: keys[3][1] = true; break;
+		case XK_f: keys[3][2] = true; break;
+		case XK_h: keys[3][3] = true; break;
+	}
+	return true;
+	return true;
+}
+
+bool
+FM_KeyReleaseEventMenuPlay(FM_Game *game, XEvent *event)
+{
+	KeySym keysym;
+
+	keysym = XLookupKeysym(&event->xkey, 0);
+	switch(keysym)
+	{
+		case XK_Up: keys[0][0] = false; break;
+		case XK_Down: keys[0][1] = false; break;
+		case XK_Left: keys[0][2] = false; break;
+		case XK_Right: keys[0][3] = false; break;
+		case XK_w: keys[1][0] = false; break;
+		case XK_s: keys[1][1] = false; break;
+		case XK_a: keys[1][2] = false; break;
+		case XK_d: keys[1][3] = false; break;
+		case XK_i: keys[2][0] = false; break;
+		case XK_k: keys[2][1] = false; break;
+		case XK_j: keys[2][2] = false; break;
+		case XK_l: keys[2][3] = false; break;
+		case XK_t: keys[3][0] = false; break;
+		case XK_g: keys[3][1] = false; break;
+		case XK_f: keys[3][2] = false; break;
+		case XK_h: keys[3][3] = false; break;
+		case XK_Escape: FM_ExitMenuPlay(game); FM_EnterMenuMain(game); break;
+	}
+	return true;
+}
+
 bool
 FM_UpdateMenuPlay(FM_Game *game)
 {
@@ -358,37 +405,13 @@ FM_UpdateMenuPlay(FM_Game *game)
 
 	for(i = 0; i < 16; i++)
 	{
-		dSpaceCollide(space, NULL, &FM_NearCallback);
-		dWorldStep(world, 0.01);
-		dJointGroupEmpty(contactgroup);
+		dSpaceCollide(game->play.space, game, &FM_NearCallback);
+		dWorldStep(game->play.world, 0.01);
+		dJointGroupEmpty(game->play.contactgroup);
 	}
 
 	FM_RenderMenuPlay(game);
 
-	return false;
+	return true;
 }
 
-bool
-FM_EventMenuPlay(FM_Game *game, SDL_Event *event)
-{
-	bool exitrequest = false;
-	switch(event->type)
-	{
-		case SDL_KEYUP:
-			switch(event->key.keysym.sym)
-			{
-				case SDLK_ESCAPE:
-					FM_ExitMenuPlay(game);
-					FM_EnterMenuMain(game);
-					break;
-				default:
-					break;
-			}
-			break;
-		case SDL_QUIT:
-			FM_ExitMenuPlay(game);
-			exitrequest = true;
-			break;
-	}
-	return exitrequest;
-}
