@@ -17,8 +17,6 @@
 #ifndef MENUPLAY_H
 #define MENUPLAY_H
 
-#include <SDL/SDL.h>
-#include <stdbool.h>
 #include "main.h"
 
 void
@@ -31,6 +29,9 @@ bool
 FM_UpdateMenuPlay(FM_Game *game);
 
 bool
-FM_EventMenuPlay(FM_Game *game, SDL_Event *event);
+FM_KeyPressEventMenuPlay(FM_Game *game, XEvent *event);
+
+bool
+FM_KeyReleaseEventMenuPlay(FM_Game *game, XEvent *event);
 
 #endif
