@@ -16,8 +16,8 @@
 
 #include "track.h"
 #include "main.h"
-#include "btm.h"
 #include "ml.h"
+#include "btm.h"
 
 static
 void
@@ -131,7 +131,7 @@ FM_UpdateTrack(FM_Track *track, FM_Machine *machine)
 }
 
 void
-FM_RenderTrack(const FM_Track *track, const RILcamera *camera)
+FM_RenderTrack(const FM_Track *track, const FM_Camera *camera)
 {
 	btmRender(&track->model);
 //	btmRenderWithCamera(camera, &track->model);
