@@ -17,12 +17,13 @@
 #ifndef TRACK_H
 #define TRACK_H
 
-#include "btm.h"
+#include "freemachinestypes.h"
 #include "machine.h"
+#include "camera.h"
+#include "btm.h"
 #include <ode/ode.h>
-#include "rilcamera.h"
 
-typedef struct
+struct FM_Track
 {
 	BTMmodel model;
 	dGeomID geom;
@@ -31,7 +32,7 @@ typedef struct
 	dTriMeshDataID trimeshdata;
 	unsigned int ncheckpoints;
 	float *checkpoints;
-} FM_Track;
+};
 
 FM_Track *
 FM_LoadTrack(dSpaceID space, const char *name);
@@ -43,6 +44,6 @@ void
 FM_UpdateTrack(FM_Track *track, FM_Machine *machine);
 
 void
-FM_RenderTrack(const FM_Track *track, const RILcamera *camera);
+FM_RenderTrack(const FM_Track *track, const FM_Camera *camera);
 
 #endif
